require 'rails_helper'

module Mks
  module Rent
    RSpec.describe ClientsController, type: :controller do
      routes { Mks::Rent::Engine.routes }

      before(:each) do
        u = create(:user)
        token = Mks::Auth::TokenAuth.issue(name: u.full_name, email: u.email, id: u.id)
        request.headers['Authorization'] = "Bearer #{token}"

        @client = create(:client)
      end

      let(:valid_attributes) do
        {
          name: FFaker::Name.name,
          address: FFaker::Name.name
        }
      end

      let(:invalid_attributes) do
        {
          name: nil,
          address: FFaker::Name.name
        }
      end

      describe 'GET #index' do
        it 'returns a success response' do
          create(:client)
          get :index
          expect(response).to be_success
        end
      end

      describe 'GET #show' do
        it 'returns a success response' do
          client = create(:client)
          get :show, params: { id: client.to_param }
          expect(response).to be_success
        end
      end

      describe 'POST #create' do
        context 'with valid params' do
          it 'creates a new Client' do
            expect do
              post :create, params: { client: valid_attributes }
            end.to change(Client, :count).by(1)
          end

          it 'renders a JSON response with the new client' do
            post :create, params: { client: valid_attributes }
            expect(response).to have_http_status(:created)
            expect(response.content_type).to eq('application/json')
            expect(response.location).to eq(client_url(Client.last))
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the new client' do
            post :create, params: { client: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

      describe 'PUT #update' do
        context 'with valid params' do
          let(:new_attributes) do
            {
              name: FFaker::Name.name
            }
          end

          it 'updates the requested client' do
            client = create(:client)
            put :update, params: { id: client.to_param, client: new_attributes }
            client.reload
            expect(client.name).to eq new_attributes[:name]
          end

          it 'renders a JSON response with the client' do
            client = create(:client)
            put :update, params: { id: client.to_param, client: valid_attributes }
            expect(response).to have_http_status(:ok)
            expect(response.content_type).to eq('application/json')
          end
        end

        context 'with invalid params' do
          it 'renders a JSON response with errors for the client' do
            client = create(:client)
            put :update, params: { id: client.to_param, client: invalid_attributes }
            expect(response).to have_http_status(:unprocessable_entity)
            expect(response.content_type).to eq('application/json')
          end
        end
      end

    end
  end
end